@@ -32,11 +32,11 @@ class SAM(object):
 
     Parameters
     ----------
-    filename : string
-        The path to the data file. 
+    counts : pandas.DataFrame, optional, default None
+    	A DataFrame of the gene expression counts (cells x genes).
 
-    ann_name : string, optional, default None
-        The path to the cell annotation file, should one exist.
+    annotations : numpy.ndarray, optional, default None
+        A Numpy array of cell annotations.
 
     k : int, optional, default None
         The number of nearest neighbors to identify for each cell. If None,
@@ -49,16 +49,14 @@ class SAM(object):
         
     Attributes
     ----------
-    filename: The path to the data file.
+    dataset : pandas.DataFrame of the unfiltered raw expression counts (cells x genes)
 
-    ann_name: The path to the cell annotations file (optional).
+    ann : numpy.ndarray of the cell annotations (optional).
 
     k: The number of nearest neighbors to identify for each cell when constructing the nearest neighbor graph.
 
     distance: The distance metric used when constructing the cell-to-cell distance matrix.
 
-    dataset: A Pandas DataFrame containing the original input data (cells x genes).
-
     filtered_dataset: A Pandas DataFrame containing the filtered data (cells x genes).
 
     num_expressed_genes: The number of expressed genes in each cell.
@@ -76,7 +74,11 @@ class SAM(object):
     D_avg: The k-nearest-neighbor-averaged expression data.
 
     weighted_data: The rescaled expression data.
+    
+    D_sub: If the 'n_genes' parameter is set, D_sub is the subsetted gene expression matrix. Otherwise, it is equal to 'D'.
 
+    weighted_sub: If the 'n_genes' parameter is set, weighted_sub is the subsetted weighted expression matrix. Otherwise, it is equal to 'weighted_data'.
+    
     wPCA_data: The weighted PCA matrix.
 
     pca: The sklearn pca object.
@@ -99,24 +101,45 @@ class SAM(object):
     
     """
     
-    def __init__(self,filename,ann_name=None,k=None,distance='correlation'):
-        self.filename=filename
-        self.ann_name=ann_name
+    def __init__(self,counts=None,annotations=None,k=None,distance='correlation'):
+        self.dataset=counts
+        self.ann = annotations
+        
+        if(self.ann is not None):
+            self.ann_int=ut.convert_annotations(self.ann)
+            
+        if(self.dataset is not None):
+            self.filtered_dataset = self.remove_zero_columns(self.dataset)
+            self.log_transform(self.filtered_dataset)
+            self.load_attributes_from_data(self.filtered_dataset)
+        
         self.k=k
         self.distance=distance
         self.analysis_performed=False
   
-    def load_data(self,sep=',',**kwargs):
-        """Reads the specified file and stores the data in a Pandas DataFrame.
+    def load_data_from_file(self,filename,do_filtering=True,transpose=True,sep=',',**kwargs):
+        """Reads the specified tabular data file and stores the data in a Pandas DataFrame.
 
-        This is a wrapper function that loads the file specified by 'filename', 
-        filters the data, and loads cell annotations if 'ann_name' is not None.
+        This is a wrapper function that loads the file specified by 'filename' and 
+        filters the data.
         
         Parameters
         ----------
+        filename - string
+            The path to the tabular raw expression counts file.
+            
         sep - string, optional, default ','
             The delimeter used to read the input data table. By default assumes the input table is delimited by commas.
         
+        do_filtering - bool, optional, default True
+            If True, filters the data with default parameters using 'filter_data'.
+            Otherwise, loads the data without filtering (aside from removing genes
+            with no expression at all).
+            
+        transpose - bool, optional, default True
+            By default, assumes file is (genes x cells). Set this to False if the file
+            has dimensions (cells x genes)
+            
         Keyword arguments
         -----------------
         
@@ -153,11 +176,34 @@ class SAM(object):
             filtering operations. 
        
         """
-        df = pd.read_csv(self.filename,sep=sep, index_col=0) 
-        self.dataset=df.T  
-        self.filter_data(**kwargs)
-        self.load_annotations()        
+        df = pd.read_csv(filename,sep=sep, index_col=0) 
+        if(transpose):
+            self.dataset=df.T  
+        else:
+            self.dataset=df
+        
+        if(do_filtering):
+            self.filter_data(**kwargs)
+        else:
+            self.filtered_dataset=self.remove_zero_columns(self.dataset)
+            self.log_transform(self.filtered_dataset)
+            self.load_attributes_from_data(self.filtered_dataset)
     
+    def remove_zero_columns(self,data):
+        return data.iloc[:,data.values.sum(0)>0]
+    
+    def load_attributes_from_data(self,data):    
+        self.D=data.values.copy()
+        self.gene_names=np.array(list(data.columns.values))
+        self.cell_names=np.array(list(data.index.values))
+
+    def log_transform(self,data,in_place=True):
+        if(in_place):
+            np.add(data.values,1,out=data.values);
+            np.log2(data.values,out=data.values);
+        else:
+            return np.log2(data+1)
+        
     def filter_data(self,div=1,downsample=0,genes=None,cells=None,min_expression=1,thresh=0.02,filter_genes=True):              
         """Log-normalizes and filters the expression data.
         
@@ -197,7 +243,10 @@ class SAM(object):
             filtering operations. 
        
         """
-        self.filtered_dataset=np.log2(self.dataset/div+1)
+        if(self.dataset is None):
+            print('No data loaded')
+            return
+        self.filtered_dataset=self.log_transform(self.dataset/div,in_place=False)
         
         if(genes is not None):
             genes=np.array(genes)
@@ -245,41 +294,31 @@ class SAM(object):
             
         self.filtered_dataset = self.filtered_dataset.iloc[:,keep]        
         self.filtered_dataset[self.filtered_dataset<=min_expression]=0        
-
-        self.D=self.filtered_dataset.values
-
-        self.gene_names=np.array(list(self.filtered_dataset.columns.values))
-        self.cell_names=np.array(list(self.filtered_dataset.index.values))
+        
+        self.load_attributes_from_data(self.filtered_dataset)
+        
      
-    def load_annotations(self,ann_name=None):   
+    def load_annotations(self,aname):   
         """Loads cell annotations.
         
-        Loads the cell annoations specified by 'ann_name' during the creation
-        of the SAM object.
+        Loads the cell annoations specified by the 'aname' path.
 
-        """      
-        if(ann_name is None):
-            aname = self.ann_name        
+        """     
+        ann = pd.read_csv(aname,header=None)                
+        if(ann.size!=self.dataset.shape[0]):
+            ann = pd.read_csv(aname,index_col=0,header=None)
+        if(ann.size!=self.dataset.shape[0]):
+            ann = pd.read_csv(aname,index_col=0)
+        if(ann.size!=self.dataset.shape[0]):
+            ann = pd.read_csv(aname)
+
+        if(ann.size!=self.filtered_dataset.shape[0]):
+            ann=ann.values.flatten()[np.where(np.in1d(self.dataset.index.values,self.filtered_dataset.index.values))[0]]
         else:
-            aname=ann_name
+            ann=ann.values.flatten()
 
-
-        if(aname): 
-            ann = pd.read_csv(aname,header=None)                
-            if(ann.size!=self.dataset.shape[0]):
-                ann = pd.read_csv(aname,index_col=0,header=None)
-            if(ann.size!=self.dataset.shape[0]):
-                ann = pd.read_csv(aname,index_col=0)
-            if(ann.size!=self.dataset.shape[0]):
-                ann = pd.read_csv(aname)
-                                       
-            if(ann.size!=self.filtered_dataset.shape[0]):
-                ann=ann.values.flatten()[np.where(np.in1d(self.dataset.index.values,self.filtered_dataset.index.values))[0]]
-            else:
-                ann=ann.values.flatten()
-            
-            self.ann=ann
-            self.ann_int=ut.convert_annotations(self.ann)  
+        self.ann=ann
+        self.ann_int=ut.convert_annotations(self.ann)  
             
     def dispersion_ranking_NN(self,dist,num_norm_avg=50):
         """Computes the spatial dispersion factors for each gene.
@@ -300,6 +339,9 @@ class SAM(object):
         dist - ndarray, float
             Square cell-to-cell distance matrix.
         
+        num_norm_avg - int, optional, default 50
+            The top 'num_norm_avg' dispersions are averaged to determine the normalization factor
+            when calculating the weights.
         
         Returns:
         -------
@@ -334,7 +376,7 @@ class SAM(object):
         return indices,weights,nnm,D_avg
 
 
-    def run(self,max_iter=15,stopping_condition=1e-5,verbose=True,projection=True,n_genes=None,npcs=150):
+    def run(self,max_iter=15,stopping_condition=1e-5,verbose=True,projection=True,n_genes=None,npcs=150,num_norm_avg=50):
         """Runs the Self-Assembling Manifold algorithm.
         
         Parameters
@@ -364,6 +406,10 @@ class SAM(object):
         n_genes - int, optional, default None
             Improve runtime by selecting only the top 'n_genes' weighted genes
             when computing PCA. If None, use all genes.
+        
+        num_norm_avg - int, optional, default 50
+            The top 'num_norm_avg' dispersions are averaged to determine the normalization factor
+            when calculating the weights.
         """  
         if(not self.k):            
             self.k = int(self.D.shape[0]**0.5)
@@ -373,12 +419,16 @@ class SAM(object):
         elif(self.k>100):
             self.k = 100
         
+        if(n_genes is not None):
+            if(n_genes < 2*num_norm_avg):
+                n_genes=2*num_norm_avg
+            
         if(self.k > self.D.shape[0]-1):
             print("Warning: chosen k exceeds the number of cells")
             self.k = self.D.shape[0]-2
         
         
-        print('RUNNING ' + self.filename)        
+        print('RUNNING SAM')        
         
         numcells=self.D.shape[0]
         tinit=time.time()
@@ -388,7 +438,7 @@ class SAM(object):
         dist[np.arange(numcells),np.arange(numcells)]=0
 
 
-        _,dispersions,edm,_=self.dispersion_ranking_NN(dist)
+        _,dispersions,edm,_=self.dispersion_ranking_NN(dist,num_norm_avg=num_norm_avg)
         
         W=dispersions.reshape((1,self.D.shape[1]))
 
@@ -423,7 +473,7 @@ class SAM(object):
             self.pca=pca
 
             dist=ut.compute_distances(g_weighted,self.distance)
-            idx2,dispersions,EDM,_=self.dispersion_ranking_NN(dist)
+            idx2,dispersions,EDM,_=self.dispersion_ranking_NN(dist,num_norm_avg=num_norm_avg)
 
             W = dispersions.reshape((1,self.D.shape[1]))
 
